@@ -24,30 +24,17 @@ compile_error!("A clockspeed feature (\"16MHz\", \"20MHz\") must be enabled for
 #[cfg(not(any(feature="atmega4809", feature="atmega328p")))]
 compile_error!("A processor feature (\"atmega4809\", \"atmega328p\") must be enabled for this crate");
 
-#[cfg(not(any(feature="alloc_small",feature="alloc_medium",feature="alloc_large")))]
-compile_error!("An allocator feature (\"alloc_small\", \"alloc_medium\", \"alloc_large\") must be enabled for this crate");
-
 #[cfg(feature = "atmega4809")]
 pub mod memory {
   pub const FLASH_SIZE  : usize  = 49_152;
   pub const SRAM_SIZE   : usize  =  6_144;
+  pub const SRAM_END    : usize  = 0x3FFF;
   pub const EEPROM_SIZE : usize  =    256;
   pub const USER_SIZE   : usize  =     64;
 
-  #[cfg(feature = "alloc_small")]
   pub mod alloc {
-    pub        const HEAPSIZE : usize = 512;
-    pub(crate) const SMAX     : usize = 8;
-  }
-  #[cfg(feature = "alloc_medium")]
-  pub mod alloc {
-    pub        const HEAPSIZE : usize = 1024;
-    pub(crate) const SMAX     : usize = 16;
-  }
-  #[cfg(feature = "alloc_large")]
-  pub mod alloc {
-    pub        const HEAPSIZE : usize = 3072;
-    pub(crate) const SMAX     : usize = 32;
+    pub const MIN_HEAPSIZE : usize = 2560;
+    pub const SMAX         : usize = 128;
   }
 
   pub mod eeprom {
@@ -60,22 +47,12 @@ pub mod memory {
 pub mod memory {
   pub const FLASH_SIZE  : usize  = 32_768;
   pub const SRAM_SIZE   : usize  =  2_048;
+  pub const SRAM_END    : usize  = 0x08FF;
   pub const EEPROM_SIZE : usize  =  1_024;
 
-  #[cfg(feature = "alloc_small")]
   pub mod alloc {
-    pub        const HEAPSIZE : usize = 128;
-    pub(crate) const SMAX     : usize = 8;
-  }
-  #[cfg(feature = "alloc_medium")]
-  pub mod alloc {
-    pub        const HEAPSIZE : usize = 256;
-    pub(crate) const SMAX     : usize = 8;
-  }
-  #[cfg(feature = "alloc_large")]
-  pub mod alloc {
-    pub        const HEAPSIZE : usize = 512;
-    pub(crate) const SMAX     : usize = 8;
+    pub const MIN_HEAPSIZE : usize = 768;
+    pub const SMAX         : usize = 64;
   }
 
   pub mod eeprom {
@@ -83,10 +60,29 @@ pub mod memory {
   }
 }
 
+#[cfg(feature="atmega4809")]
 pub mod oxide {
-  pub const MAX_THREADS : usize  = 1;
+  pub const MAX_THREADS       : usize  = 6;
+  pub const DEFAULT_MAIN_THREAD_STACK_SIZE : usize = 512;
+  pub const DEFAULT_THREAD_STACK_SIZE : usize = 128;
+
+  pub const PIN_ARRAY_SIZE    : usize = 48;
+  pub const SERIAL_BUFFER     : usize = 32;
+  pub const EVENT_QUEUE       : usize = 16;
 }
 
+#[cfg(feature="atmega328p")]
+pub mod oxide {
+  pub const MAX_THREADS       : usize  = 4;
+  pub const DEFAULT_MAIN_THREAD_STACK_SIZE : usize = 384;
+  pub const DEFAULT_THREAD_STACK_SIZE : usize = 64;
+
+  pub const PIN_ARRAY_SIZE    : usize = 24;
+  pub const SERIAL_BUFFER     : usize = 8;
+  pub const EVENT_QUEUE       : usize = 8;
+}
+
+
 #[cfg(feature="20MHz")]
 pub mod clock {
   pub const MASTER_CLOCK_HZ: u32         = 20_000_000; // Config'd in device fuses
