@@ -20,32 +20,29 @@
 //! # #![no_std]
 //! # #![no_main]
 //! #
-//! # use avr_oxide::hal::atmega4809::hardware;
 //! # use avr_oxide::alloc::boxed::Box;
 //! # use avr_oxide::devices::UsesPin;
 //! # use avr_oxide::devices::debouncer::Debouncer;
 //! # use avr_oxide::devices::{ Handle, OxideButton, button::ButtonState };
-//! # use avr_oxide::arduino;
+//! # use avr_oxide::boards::board;
 //!
 //! #[avr_oxide::main(chip="atmega4809")]
 //! pub fn main() {
 //!   let supervisor = avr_oxide::oxide::instance();
-//!   let hardware = hardware::instance();
-//!   let arduino = arduino::nanoevery::Arduino::from(hardware);
 //!
-//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(arduino.a2)));
+//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(board::pin_a(2))));
 //!
 //!   green_button.on_click(Box::new(move |_pinid, state|{
 //!     match state {
 //!       ButtonState::Released => {
 //!         // Do something
-//!       },
+//!       }
 //!       ButtonState::Pressed => {
 //!         // Do something
-//!       },
+//!       }
 //!       ButtonState::Unknown => {
 //!         // Probably don't do anything
-//!       },
+//!       }
 //!     }
 //!   }));
 //!
@@ -63,16 +60,16 @@ use avr_oxide::hal::generic::port::{Pin, PinMode, InterruptMode, PinIdentity};
 use avr_oxide::event::{EventSink, EventSource, OxideEvent, OxideEventEnvelope};
 use core::marker::PhantomData;
 use core::ops::DerefMut;
-use core::cell::RefCell;
+use core::cell::{RefCell, UnsafeCell};
 use ufmt::derive::uDebug;
 use avr_oxide::devices::internal::StaticShareable;
 use oxide_macros::Persist;
 use avr_oxide::hal::generic::callback::IsrCallback;
-#[cfg(feature="alloc_ftr")]
 use avr_oxide::alloc::boxed::Box;
 use avr_oxide::devices::UsesPin;
-use avr_oxide::panic_if_none;
-use avr_oxide::util::OwnOrBorrowMut;
+use avr_oxide::halt_if_none;
+use avr_oxide::private::binaryringq::BinaryRingQ;
+use avr_oxide::util::OwnOrBorrow;
 
 // Declarations ==============================================================
 
@@ -93,52 +90,49 @@ pub enum ButtonState {
  * Encapsulation of a Button attached to a standard GPIO pin.  Generates Oxide
  * events on button events (press/release.)
  */
-pub struct Button<'b, P, S>
+pub struct Button<'b, S>
 where
-  P: 'static + Pin,
   S: EventSink
 {
-  pin: OwnOrBorrowMut<'static,P>,
+  pin: OwnOrBorrow<'static,dyn Pin>,
   phantom: PhantomData<S>,
 
-  on_click: RefCell<Option<Box<dyn FnMut(PinIdentity,ButtonState)->() + 'b>>>
+  on_click: RefCell<Option<Box<dyn FnMut(PinIdentity,ButtonState)->() + 'b>>>,
+  wait_events: UnsafeCell<BinaryRingQ>
 }
 
-impl<'b,P,S> StaticShareable for Button<'b, P, S>
+impl<'b,S> StaticShareable for Button<'b, S>
   where
-    P: 'static + Pin,
     S: EventSink
 {}
 
 // Code ======================================================================
-impl<'b,P,S> UsesPin<P> for Button<'b,P,S>
+impl<'b,S> UsesPin for Button<'b,S>
 where
-  P: Pin,
   S: 'static + EventSink
 {
-  fn using<OP: Into<OwnOrBorrowMut<'static, P>>>(pin: OP) -> Self {
-    let pin : OwnOrBorrowMut<P> = pin.into();
+  fn using<OP: Into<OwnOrBorrow<'static, dyn Pin>>>(pin: OP) -> Self {
+    let pin : OwnOrBorrow<dyn Pin> = pin.into();
 
-    pin.set_mode(PinMode::InputFloating);
+    pin.set_mode(PinMode::Input);
     pin.set_interrupt_mode(InterruptMode::BothEdges);
     Button {
       pin,
       phantom: PhantomData::default(),
-      on_click: RefCell::new(None)
+      on_click: RefCell::new(None),
+      wait_events: UnsafeCell::new(BinaryRingQ::new())
     }
   }
 }
 
-impl<'b,P,S> Button<'b,P,S>
+impl<'b,S> Button<'b,S>
 where
-  P: Pin,
   S: 'static + EventSink
 {
   /**
    * Have the Oxide supervisor execute the given closure every time an event
    * is generated by this device.
    */
-  #[cfg(feature="alloc_ftr")]
   pub fn on_click(&self, bf: Box<dyn FnMut(PinIdentity, ButtonState) -> () + 'b>) {
     self.on_click.replace(Some(bf));
   }
@@ -154,31 +148,62 @@ where
     self
   }
 
+  /**
+   * True if and only if this button is currently pressed
+   */
   pub fn is_pressed(&self) -> bool {
     !self.pin.get()
   }
 
+  /**
+   * Return the current state of this button.
+   */
   pub fn state(&self) -> ButtonState {
     match self.pin.get() {
-      true => ButtonState::Pressed,
+      true  => ButtonState::Pressed,
+      false => ButtonState::Released
+    }
+  }
+
+  /**
+   * Block until the state of this button changes.  The new state is
+   * returned.
+   */
+  pub fn wait_for_change(&self) -> ButtonState {
+    match unsafe {
+      let wait_events = &mut *self.wait_events.get();
+
+      wait_events.consume_blocking()
+    } {
+      true  => ButtonState::Pressed,
       false => ButtonState::Released
     }
   }
 }
 
-impl<P,S> EventSource for Button<'_,P,S>
+impl<S> EventSource for Button<'_,S>
 where
-  P: Pin,
   S: EventSink
 {
   fn listen(&'static self) {
-    self.pin.listen(IsrCallback::WithData(|_source, id, state, udata|{
-      S::event(OxideEventEnvelope::to(unsafe { &*(panic_if_none!(udata) as *const Button<P,S> as *const dyn EventSource) },
-                                      OxideEvent::ButtonEvent(id,
-                                                              match state {
-                                                                true  => ButtonState::Pressed,
-                                                                false => ButtonState::Released
-                                                              })));
+    self.pin.listen(IsrCallback::WithData(|isotoken, _source, id, state, udata|{
+      // This is executing within the ISR context
+      //
+      // First, release any threads waiting for an event
+      unsafe {
+        let myself = halt_if_none!(udata, avr_oxide::oserror::OsError::InternalError) as *const Button<S>;
+        let wait_events = &mut *((*myself).wait_events.get());
+
+        let _ = wait_events.append(isotoken, state);
+
+        // Then also send an OxideEvent
+        S::event(isotoken, OxideEventEnvelope::to(&*(myself as *const dyn EventSource),
+                                                  OxideEvent::ButtonEvent(id,
+                                                                          match state {
+                                                                            true  => ButtonState::Pressed,
+                                                                            false => ButtonState::Released
+                                                                          })));
+      }
     }, self as *const dyn core::any::Any ))
   }
 
@@ -192,3 +217,12 @@ where
   }
 }
 
+unsafe impl <S> Send for Button<'_,S>
+where
+  S: EventSink
+{}
+
+unsafe impl <S> Sync for Button<'_,S>
+  where
+    S: EventSink
+{}
\ No newline at end of file
