@@ -14,18 +14,15 @@
 //! # #![no_std]
 //! # #![no_main]
 //! #
-//! # use avr_oxide::hal::atmega4809::hardware;
 //! # use avr_oxide::devices::UsesPin;
 //! # use avr_oxide::devices::OxideLed;
-//! # use avr_oxide::arduino;
+//! # use avr_oxide::boards::board;
 //!
 //! # #[avr_oxide::main(chip="atmega4809")]
 //! # pub fn main() {
 //! #   let supervisor = avr_oxide::oxide::instance();
-//! #   let hardware = hardware::instance();
-//!   let arduino = arduino::nanoevery::Arduino::from(hardware);
 //!
-//!   let green_led = OxideLed::with_pin(arduino.d7);
+//!   let green_led = OxideLed::with_pin(board::pin_d(7));
 //!
 //!   green_led.toggle();
 //! #
@@ -36,31 +33,22 @@
 
 // Imports ===================================================================
 use avr_oxide::hal::generic::port::{Pin, PinMode, InterruptMode};
-use avr_oxide::util::OwnOrBorrowMut;
+use avr_oxide::util::OwnOrBorrow;
 use avr_oxide::devices::UsesPin;
 use avr_oxide::devices::internal::StaticShareable;
 
 
 // Declarations ==============================================================
-pub struct Led<P>
-where
-  P: 'static + Pin
-{
-  pin: OwnOrBorrowMut<'static,P>
+pub struct Led {
+  pin: OwnOrBorrow<'static,dyn Pin>
 }
 
-impl<P> StaticShareable for Led<P>
-where
-  P: 'static + Pin
-{}
+impl StaticShareable for Led {}
 
 // Code ======================================================================
-impl<P> UsesPin<P> for Led<P>
-where
-  P: Pin
-{
-  fn using<OP: Into<OwnOrBorrowMut<'static, P>>>(pin: OP) -> Self {
-    let pin : OwnOrBorrowMut<P> = pin.into();
+impl UsesPin for Led {
+  fn using<OP: Into<OwnOrBorrow<'static, dyn Pin>>>(pin: OP) -> Self {
+    let pin : OwnOrBorrow<dyn Pin> = pin.into();
 
     pin.set_mode(PinMode::Output);
     pin.set_interrupt_mode(InterruptMode::Disabled);
@@ -68,10 +56,7 @@ where
   }
 }
 
-impl<P> Led<P>
-where
-  P: Pin
-{
+impl Led {
   pub fn set_on(&self) {
     self.pin.set_high();
   }
@@ -91,3 +76,6 @@ where
     self.pin.toggle()
   }
 }
+
+unsafe impl Send for Led {}
+unsafe impl Sync for Led {}
\ No newline at end of file
