@@ -41,9 +41,9 @@ pub mod panicout {
 #[cfg(feature="usart0")]
 pub mod usart0 {
   use avr_oxide::{atmel_0series_usart_tpl};
-  use avr_oxide::hal::generic::serial::base::zeroseries::MuxControl;
   #[cfg(target_arch="avr")]
   use avr_oxide::hal::atmega4809::cpu::PortmuxControl;
+  use avr_oxide::hal::generic::MuxControl;
 
   pub enum SerialPortPins {
     PortAlow,
@@ -52,7 +52,6 @@ pub mod usart0 {
   }
 
 
-
   impl MuxControl for SerialPortPins {
     #[cfg(target_arch="avr")]
     fn route(&self) {
@@ -89,9 +88,9 @@ pub mod usart0 {
 #[cfg(feature="usart1")]
 pub mod usart1 {
   use avr_oxide::{atmel_0series_usart_tpl};
-  use avr_oxide::hal::generic::serial::base::zeroseries::MuxControl;
   #[cfg(target_arch="avr")]
   use avr_oxide::hal::atmega4809::cpu::PortmuxControl;
+  use avr_oxide::hal::generic::MuxControl;
 
   pub enum SerialPortPins {
     PortClow,
@@ -131,9 +130,9 @@ pub mod usart1 {
 #[cfg(feature="usart2")]
 pub mod usart2 {
   use avr_oxide::{atmel_0series_usart_tpl};
-  use avr_oxide::hal::generic::serial::base::zeroseries::MuxControl;
   #[cfg(target_arch="avr")]
   use avr_oxide::hal::atmega4809::cpu::PortmuxControl;
+  use avr_oxide::hal::generic::MuxControl;
 
   pub enum SerialPortPins {
     PortFlow,
@@ -175,9 +174,9 @@ pub mod usart2 {
 #[cfg(feature="usart3")]
 pub mod usart3 {
   use avr_oxide::{atmel_0series_usart_tpl};
-  use avr_oxide::hal::generic::serial::base::zeroseries::MuxControl;
   #[cfg(target_arch="avr")]
   use avr_oxide::hal::atmega4809::cpu::PortmuxControl;
+  use avr_oxide::hal::generic::MuxControl;
 
   pub enum SerialPortPins {
     PortBlow,
