@@ -12,10 +12,9 @@
 //! with the Persistent trait.
 //!
 //! # Usage
-//! ```rust
+//! ```rust,no_run
 //! # use oxide_macros::Persist;
 //! # use avr_oxide::util::persist::Persist;
-//! # use avr_oxide::hal::atmega4809::hardware;
 //! # use avr_oxide::hal::generic::eeprom::EepromSpace;
 //! # use avr_oxide::panic_if_err;
 //! #
@@ -26,8 +25,7 @@
 //! }
 //!
 //! pub fn load_save_object() -> ! {
-//!   let hardware = hardware::instance();
-//!   let eeprom = &mut hardware.eeprom;
+//!   let eeprom = &mut avr_oxide::hardware::nvm::eeprom::instance();
 //!
 //!   let mut thing : MyDataStructure = panic_if_err!(Persist::load_with(eeprom.reader()));
 //!   thing.number += 1u8;
@@ -89,10 +87,11 @@ pub mod base {
   pub mod nvmctrl {
     use avr_oxide::devices::internal::StaticShareable;
     use avr_oxide::hal::generic::eeprom::EepromSpace;
-    use avr_oxide::{v_read, v_write};
-    use avr_oxide::hal::generic::cpu::base::AvrCpuControlBlock;
+    use avr_oxide::cpu;
     use avr_oxide::hal::generic::cpu::{ConfigurationChange, Cpu};
     use avr_oxide::io::{Read, Write};
+    use avr_oxide::sync::Mutex;
+    use avr_oxide::util::datatypes::Volatile;
 
     #[repr(C)]
     pub struct AtmelNvmCtrl {
@@ -109,10 +108,9 @@ pub mod base {
     }
 
     pub struct EepromNvmCtrlImpl<const SIZE: usize, const PAGESIZE: usize, const PAGES: usize> {
-
       pub(crate) control: &'static mut AtmelNvmCtrl,
-      pub(crate) buffer:  &'static mut [u8; SIZE],
-      pub(crate) cpu:     &'static mut AvrCpuControlBlock,
+      pub(crate) buffer:  &'static mut [Volatile<u8>; SIZE],
+      pub(crate) mutex: Mutex<()>
     }
 
     pub struct EepromNvmCtrlReader<'r,const SIZE: usize, const PAGESIZE: usize, const PAGES: usize> {
@@ -133,6 +131,8 @@ pub mod base {
       type R = EepromNvmCtrlReader<'e,SIZE,PAGESIZE,PAGES>;
 
       fn read_at_into(&self, offset: usize, dest: &mut [u8]) -> usize {
+        let lock = self.mutex.lock();
+
         let mut bytes_read : usize = 0;
 
         for idx in 0..dest.len() {
@@ -141,15 +141,17 @@ pub mod base {
           if loc >= SIZE {
             break;
           } else {
-            dest[idx] = unsafe { v_read!(u8, self.buffer[loc]) };
+            dest[idx] = self.buffer[loc].read();
             bytes_read += 1;
           }
         }
 
+        drop(lock);
         bytes_read
       }
 
       fn write_at_from(&mut self, offset: usize, src: &[u8]) -> usize {
+        let lock = self.mutex.lock();
         let mut bytes_consumed : usize = 0;
 
         // Copy down backwards so we can find the page boundaries neatly
@@ -158,15 +160,15 @@ pub mod base {
           let loc = offset+idx;
 
           if loc >= SIZE { // Don't do this, please
-            panic!()
+            avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams);
           }
 
           // Read the given byte from the page buffer.  This will also force
           // the chip to wait until any pending writes are complete, so we
           // don't need to check the status separately.
-          let current_val = unsafe { v_read!(u8, self.buffer[loc]) };
+          let current_val = self.buffer[loc].read();
           if current_val != src[idx] {
-            unsafe { v_write!(u8, self.buffer[loc], src[idx]) };
+            self.buffer[loc].write(src[idx]);
             page_dirty = true;
           }
           bytes_consumed += 1;
@@ -175,7 +177,7 @@ pub mod base {
             if page_dirty {
               // Instruct the chip to write the page
               unsafe {
-                self.cpu.write_protected(ConfigurationChange::SelfProgramming,
+                cpu!().write_protected(ConfigurationChange::SelfProgramming,
                                          &mut self.control.ctrla, 0x03); // 0x03 == ERWP (Erase and Write Page)
               }
             }
@@ -183,6 +185,7 @@ pub mod base {
           }
         }
 
+        drop(lock);
         bytes_consumed
       }
 
@@ -238,23 +241,28 @@ pub mod base {
       }
     }
 
+    unsafe impl<const SIZE: usize, const PAGESIZE: usize, const PAGES: usize> Send for EepromNvmCtrlWriter<'_,SIZE,PAGESIZE,PAGES> {}
+    unsafe impl<const SIZE: usize, const PAGESIZE: usize, const PAGES: usize> Sync for EepromNvmCtrlWriter<'_,SIZE,PAGESIZE,PAGES> {}
+
     #[doc(hidden)]
     #[macro_export]
     macro_rules! atmel_nvmctrl_eeprom_tpl {
       ($nvmctrlref:expr, $pagebufferref:expr, $pagesize:expr, $pages:expr, $cpu:expr) => {
         use avr_oxide::hal::generic::eeprom::base::nvmctrl::EepromNvmCtrlImpl;
-        use avr_oxide::mut_singleton;
+        use avr_oxide::mut_singleton_explicit_init;
+        use avr_oxide::sync::Mutex;
+
 
         pub type EepromImpl = EepromNvmCtrlImpl<{$pagesize*$pages},{$pagesize},{$pages}>;
 
-        mut_singleton!(
+        mut_singleton_explicit_init!(
           EepromImpl,
-          INSTANCE,
-          instance,
+          __INSTANCE,
+          initialise, instance, instance_isolated,
           EepromNvmCtrlImpl {
             control: core::mem::transmute($nvmctrlref),
             buffer:  core::mem::transmute($pagebufferref),
-            cpu:     $cpu
+            mutex:   Mutex::new(())
           }
         );
       }
@@ -307,14 +315,14 @@ pub mod base {
   macro_rules! atmel_nvmctrl_eeprom_tpl {
     ($nvmctrlref:expr, $pagebufferref:expr, $pagesize:expr, $pages:expr, $cpu:expr) => {
       use avr_oxide::hal::generic::eeprom::base::DummyEepromImpl;
-      use avr_oxide::mut_singleton;
+      use avr_oxide::mut_singleton_explicit_init;
 
       pub type EepromImpl = DummyEepromImpl<{$pagesize*$pages}>;
 
-      mut_singleton!(
+      mut_singleton_explicit_init!(
         EepromImpl,
-        INSTANCE,
-        instance,
+        __INSTANCE,
+        initialise, instance, instance_isolated,
         DummyEepromImpl {
           buffer:  [ 0xff; {$pagesize*$pages} ]
         }
