@@ -15,7 +15,7 @@ pub mod watchdog;
 pub mod cpu;
 pub mod serial;
 pub mod eeprom;
-
+pub mod twi;
 
 #[lang = "eh_personality"]
 #[no_mangle]
@@ -44,3 +44,12 @@ impl<T> MutDeviceRef<T> {
   }
 }
 
+/**
+ * Trait implemented to allow us to set portmux control bits
+ */
+pub trait MuxControl {
+  /**
+   * Set the relevant hardware controls to route this appropriately.
+   */
+  fn route(&self);
+}
\ No newline at end of file
