@@ -8,6 +8,7 @@
 // Imports ===================================================================
 use core::any::Any;
 use ufmt::derive::uDebug;
+use avr_oxide::concurrency::Isolated;
 use oxide_macros::Persist;
 use avr_oxide::hal::generic::callback::IsrCallback;
 
@@ -47,7 +48,7 @@ pub enum TimerIdentity {
  * using mutual exclusion where necessary, and *DO NOT DO HEAVY PROCESSING
  * IN THE CALLBACK*.
  */
-pub type TimerIsrFunction = fn(TimerIdentity, u16, Option<*const dyn Any>) ->bool;
+pub type TimerIsrFunction = fn(Isolated, TimerIdentity, u16, Option<*const dyn Any>) ->bool;
 
 pub type TimerIsrCallback = IsrCallback<TimerIsrFunction,bool>;
 
@@ -131,88 +132,68 @@ pub trait RtcTimerCalibration {
 // Code ======================================================================
 #[cfg(target_arch="avr")]
 pub mod base {
-  use core::cell::{Cell, RefCell};
+  use core::cell::{Cell};
   use avr_oxide::hal::generic::timer::{TimerIsrCallback, TimerMode, TimerControl, TimerIdentity, RtcTimerCalibration, RtcSource, RtcCalibration, RtcPrescaler};
-  use avr_oxide::{isr_cb_invoke, v_read, v_write};
+  use avr_oxide::{isr_cb_invoke};
+  use avr_oxide::concurrency::{interrupt, Isolated};
+  use avr_oxide::hal::generic::MutDeviceRef;
+  use avr_oxide::util::datatypes::{BitFieldAccess, BitIndex, Volatile, VolatileBitField};
 
   #[repr(C)]
   pub struct AvrTypeBTimerControl {
-    pub(crate) ctrla: u8,
-    pub(crate) ctrlb: u8,
+    pub(crate) ctrla: Volatile<u8>,
+    pub(crate) ctrlb: VolatileBitField,
     pub(crate) reserved: [u8; 2],
-    pub(crate) evctrl: u8,
-    pub(crate) intctrl: u8,
-    pub(crate) intflags: u8,
-    pub(crate) status: u8,
-    pub(crate) dbgctrl: u8,
-    pub(crate) temp: u8,
-    pub(crate) cnt: u16,
-    pub(crate) ccmp: u16
+    pub(crate) evctrl: Volatile<u8>,
+    pub(crate) intctrl: VolatileBitField,
+    pub(crate) intflags: VolatileBitField,
+    pub(crate) status: Volatile<u8>,
+    pub(crate) dbgctrl: Volatile<u8>,
+    pub(crate) temp: Volatile<u8>,
+    pub(crate) cnt: Volatile<u16>,
+    pub(crate) ccmp: Volatile<u16>
   }
 
   pub trait AtmelTCB {
-    fn enable(&mut self);
-    fn disable(&mut self);
-    fn enable_interrupt(&mut self);
-    fn clear_interrupt(&mut self);
-    fn mask_interrupt(&mut self);
-    fn set_top(&mut self, top: u16);
-    fn set_periodic_mode(&mut self);
+    fn enable(&mut self, isotoken: interrupt::token::Isolated);
+    fn disable(&mut self, isotoken: interrupt::token::Isolated);
+    fn enable_interrupt(&mut self, isotoken: interrupt::token::Isolated);
+    fn clear_interrupt(&mut self, isotoken: interrupt::token::Isolated);
+    fn mask_interrupt(&mut self, isotoken: interrupt::token::Isolated);
+    fn set_top(&mut self, isotoken: interrupt::token::Isolated, top: u16);
+    fn set_periodic_mode(&mut self, isotoken: interrupt::token::Isolated);
   }
 
   impl AtmelTCB for AvrTypeBTimerControl {
-    #[inline(always)]
-    fn enable(&mut self) {
+    fn enable(&mut self, _isotoken: interrupt::token::Isolated) {
       // Flags == Run in standby, divide clock by 2, enable
-      unsafe {
-        core::ptr::write_volatile(&mut self.ctrla as *mut u8, 0b01000011);
-      }
+      self.ctrla.write(0b01000011);
+    }
+
+    fn disable(&mut self, _isotoken: interrupt::token::Isolated) {
+      self.ctrla.write(0b00000000);
+    }
+
+    fn enable_interrupt(&mut self, isotoken: interrupt::token::Isolated) {
+      self.intctrl.set_isolated(isotoken, BitIndex::bit_c(0));
     }
 
     #[inline(always)]
-    fn disable(&mut self) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.ctrla as *mut u8, 0b00000000);
-      }
+    fn clear_interrupt(&mut self, isotoken: interrupt::token::Isolated) {
+      self.intflags.set_isolated(isotoken, BitIndex::bit_c(0));
     }
 
-    #[inline(always)]
-    fn enable_interrupt(&mut self) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.intctrl as *mut u8, 0b00000001);
-      }
+    fn mask_interrupt(&mut self, isotoken: interrupt::token::Isolated) {
+      self.intctrl.clr_isolated(isotoken, BitIndex::bit_c(0));
     }
 
-    #[inline(always)]
-    fn clear_interrupt(&mut self) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.intflags as *mut u8, 0b00000001);
-      }
+    fn set_top(&mut self, isotoken: interrupt::token::Isolated, top: u16) {
+      self.cnt.write_isolated(isotoken,0x0000);
+      self.ccmp.write_isolated(isotoken,top);
     }
 
-    #[inline(always)]
-    fn mask_interrupt(&mut self) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.intctrl as *mut u8, 0b00000000);
-      }
-    }
-
-    #[inline(always)]
-    fn set_top(&mut self, top: u16) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.cnt as *mut u16, 0x0000);
-        core::ptr::write_volatile(&mut self.ccmp as *mut u16, top);
-      }
-    }
-
-    #[inline(always)]
-    fn set_periodic_mode(&mut self) {
-      unsafe {
-        let ctrlb_reg = &mut self.ctrlb as *mut u8;
-
-        let ctrlb = core::ptr::read_volatile(ctrlb_reg);
-        core::ptr::write_volatile(ctrlb_reg, ctrlb | 0b00010000);
-      }
+    fn set_periodic_mode(&mut self, isotoken: interrupt::token::Isolated) {
+      self.ctrlb.set_isolated(isotoken, BitIndex::bit_c(4));
     }
   }
 
@@ -223,9 +204,9 @@ pub mod base {
       T: 'static + AtmelTCB
   {
     pub(crate) interrupt_handler: Cell<TimerIsrCallback>,
-    pub(crate) interrupt_period: u16,
-    pub(crate) tcb: RefCell<&'static mut T>,
-    pub(crate) count_max: u16,
+    pub(crate) interrupt_period: Volatile<u16>,
+    pub(crate) tcb: MutDeviceRef<T>,
+    pub(crate) count_max: Volatile<u16>,
     pub(crate) mode: TimerMode
   }
 
@@ -234,9 +215,7 @@ pub mod base {
       T: AtmelTCB
   {
     fn set_interrupt_period(&mut self, period: u16) {
-      unsafe {
-        core::ptr::write_volatile(&mut self.interrupt_period as *mut u16, period)
-      }
+      self.interrupt_period.write(period);
     }
 
     fn set_mode(&mut self, mode: TimerMode) {
@@ -244,38 +223,42 @@ pub mod base {
     }
 
     fn set_count_max(&mut self, max: u16) {
-      self.count_max = max;
+      self.count_max.write(max);
     }
 
     fn start(&self, handler: TimerIsrCallback) {
-      let mut tcb = self.tcb.borrow_mut();
+      avr_oxide::concurrency::interrupt::isolated(|isotoken|{
+        let tcb = self.tcb.borrow_mut(isotoken);
 
-      tcb.disable();
-      match self.mode {
-        TimerMode::Periodic => {
-          tcb.set_periodic_mode();
-          tcb.set_top(self.count_max);
-        }
-      };
+        tcb.disable(isotoken);
+        match self.mode {
+          TimerMode::Periodic => {
+            tcb.set_periodic_mode(isotoken);
+            tcb.set_top(isotoken, self.count_max.read());
+          }
+        };
+
+        tcb.clear_interrupt(isotoken);
 
-      tcb.clear_interrupt();
 
-      avr_oxide::hal::concurrency::interrupt::isolated(||{
         self.interrupt_handler.replace(handler);
-      });
-      if handler.is_nop() {
-        tcb.mask_interrupt();
-      } else {
-        tcb.enable_interrupt();
-      }
 
-      tcb.enable();
+        if handler.is_nop() {
+          tcb.mask_interrupt(isotoken);
+        } else {
+          tcb.enable_interrupt(isotoken);
+        }
+
+        tcb.enable(isotoken);
+      });
     }
 
     fn stop(&self) {
-      let mut tcb = self.tcb.borrow_mut();
+      avr_oxide::concurrency::interrupt::isolated(|isotoken|{
+        let tcb = self.tcb.borrow_mut(isotoken);
 
-      tcb.disable();
+        tcb.disable(isotoken);
+      });
     }
 
     fn get_count(&self) -> u16 {
@@ -292,99 +275,92 @@ pub mod base {
       T: AtmelTCB
   {
     #[inline(always)]
-    pub(crate) fn call_interrupt(&mut self, source: TimerIdentity, ticks: u16) {
-      let continue_running = isr_cb_invoke!(self.interrupt_handler.get(), source, ticks);
+    pub(crate) fn call_interrupt(&mut self, isotoken: Isolated, source: TimerIdentity, ticks: u16) {
+      let continue_running = isr_cb_invoke!(isotoken, self.interrupt_handler.get(), source, ticks);
 
       if !continue_running {
-        self.tcb.borrow_mut().disable();
+        self.tcb.borrow_mut(isotoken).disable(isotoken);
       }
     }
 
     #[inline(always)]
-    pub(crate) fn interrupt_period(&self) -> u16 {
-      unsafe {
-        core::ptr::read_volatile(&self.interrupt_period as *const u16)
-      }
+    pub(crate) fn interrupt_period(&self, _isotoken: Isolated) -> u16 {
+      self.interrupt_period.read()
     }
 
     #[inline(always)]
-    pub(crate) fn clear_interrupt(&self) {
-      self.tcb.borrow_mut().clear_interrupt();
+    pub(crate) fn clear_interrupt(&self, isotoken: Isolated) {
+      self.tcb.borrow_mut(isotoken).clear_interrupt(isotoken);
     }
   }
 
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_tcb {
-    ($tcbref:expr, $timersrc:expr, $isr:ident) => {
+    ($tcbref:expr, $timersrc:expr, $isr:expr) => {
       use avr_oxide::hal::generic::timer::{ TimerMode, TimerIsrCallback };
       use avr_oxide::hal::generic::timer::base::{ AtmelTimer, AtmelTCB, AvrTypeBTimerControl };
-
-      use avr_oxide::mut_singleton;
-      use core::cell::{RefCell,Cell};
+      use avr_oxide::util::datatypes::Volatile;
+      use avr_oxide::mut_singleton_explicit_init;
+      use avr_oxide::hal::generic::MutDeviceRef;
+      use core::cell::Cell;
 
 
       pub type TimerImpl = AtmelTimer<AvrTypeBTimerControl>;
 
-      mut_singleton!(
+      mut_singleton_explicit_init!(
         AtmelTimer<AvrTypeBTimerControl>,
-        INSTANCE,
-        instance,
+        __INSTANCE,
+        initialise, instance, instance_isolated,
         AtmelTimer {
           interrupt_handler: Cell::new(TimerIsrCallback::Nop(false)),
-          interrupt_period: 0,
-          tcb: RefCell::new(core::mem::transmute($tcbref)),
-          count_max: 0,
+          interrupt_period: Volatile::<u16>::zero(),
+          tcb: MutDeviceRef::new($tcbref),
+          count_max: Volatile::<u16>::zero(),
           mode: TimerMode::Periodic
         });
 
 
-      #[no_mangle]
-      pub unsafe extern "avr-interrupt" fn $isr() {
-        static mut COUNT_INTS : u16 = 0;
+      #[oxide_macros::interrupt(isr=$isr)]
+      fn usart_timer_interrupt(isotoken: avr_oxide::concurrency::Isolated) {
+        static mut COUNT_INTS : Volatile<u16> = Volatile::<u16>::zero();
 
-        avr_oxide::hal::concurrency::interrupt::isr(||{
-          let counter = core::ptr::read_volatile(&COUNT_INTS as *const u16);
+        let counter = COUNT_INTS.read();
 
-          match &mut INSTANCE {
-            None => {},
-            Some(atmeltimer) => {
+        let atmeltimer = instance_isolated(isotoken);
 
-              if counter == 0 {
-                atmeltimer.call_interrupt($timersrc, atmeltimer.interrupt_period());
-                core::ptr::write_volatile(&mut COUNT_INTS as *mut u16, atmeltimer.interrupt_period())
-              } else {
-                core::ptr::write_volatile(&mut COUNT_INTS as *mut u16, counter-1);
-              }
+        if counter == 0 {
+          atmeltimer.call_interrupt(isotoken, $timersrc, atmeltimer.interrupt_period(isotoken));
+          COUNT_INTS.write_isolated(isotoken, atmeltimer.interrupt_period(isotoken));
+        } else {
+          COUNT_INTS.write_isolated(isotoken, counter-1);
+        }
 
-              atmeltimer.clear_interrupt();
-            }
-          }
-        })
+        atmeltimer.clear_interrupt(isotoken);
       }
     }
   }
 
   #[repr(C)]
   pub struct AvrRealTimeCounterControl {
-    pub(crate) ctrla: u8,
-    pub(crate) status: u8,
-    pub(crate) intctrl: u8,
-    pub(crate) intflags: u8,
-    pub(crate) temp: u8,
-    pub(crate) dbgctrl: u8,
-    pub(crate) calib: u8,
-    pub(crate) clksel: u8,
-    pub(crate) cnt: u16,
-    pub(crate) per: u16,
-    pub(crate) cmp: u16,
+    pub(crate) ctrla: Volatile<u8>,
+    pub(crate) status: VolatileBitField,
+    pub(crate) intctrl: VolatileBitField,
+    pub(crate) intflags: VolatileBitField,
+    pub(crate) temp: Volatile<u8>,
+    pub(crate) dbgctrl: Volatile<u8>,
+    pub(crate) calib: Volatile<u8>,
+    pub(crate) clksel: Volatile<u8>,
+    pub(crate) cnt: Volatile<u16>,
+    pub(crate) per: Volatile<u16>,
+    pub(crate) cmp: Volatile<u16>,
     res_1: u8,
     res_2: u8,
-    pub(crate) pitctrla: u8,
-    pub(crate) pitstatus: u8,
-    pub(crate) pitintctrl: u8,
-    pub(crate) pitintflags: u8,
-    pub(crate) pitdbgctrl: u8
+    pub(crate) pitctrla: Volatile<u8>,
+    pub(crate) pitstatus: VolatileBitField,
+    pub(crate) pitintctrl: VolatileBitField,
+    pub(crate) pitintflags: VolatileBitField,
+    pub(crate) pitdbgctrl: Volatile<u8>
   }
 
   pub enum RtcPitPeriod {
@@ -407,7 +383,7 @@ pub mod base {
   pub struct AvrRealTimeCounter {
     pub(crate) interrupt_handler: Cell<TimerIsrCallback>,
     pub(crate) interrupt_period: RtcPitPeriod,
-    pub(crate) rtc: RefCell<&'static mut AvrRealTimeCounterControl>,
+    pub(crate) rtc: MutDeviceRef<AvrRealTimeCounterControl>,
     pub(crate) mode: TimerMode
   }
 
@@ -428,7 +404,7 @@ pub mod base {
         8192 => RtcPitPeriod::Cyc8192,
         16384 => RtcPitPeriod::Cyc16384,
         32768 => RtcPitPeriod::Cyc32768,
-        _ => panic!()
+        _ => avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams)
       }
     }
   }
@@ -438,14 +414,13 @@ pub mod base {
     /// this function will also disable the clock until the next time you
     /// set it running (with, for example, the start() method :-).)
     fn set_clock_calibration(&mut self, source: RtcSource, calibration: RtcCalibration, prescaler: RtcPrescaler) {
-      let mut rtc = self.rtc.borrow_mut();
+      interrupt::isolated(|isotoken|{
+        let rtc = self.rtc.borrow_mut(isotoken);
 
-      unsafe {
         // Wait for clock control register to not be busy
-        while (v_read!(u8, rtc.status) & 0x01) != 0 {}
-        // Note: Side effect of this will be to disable the RTC.  That suits me
-        // fine.
-        v_write!(u8, rtc.ctrla, match prescaler {
+        while rtc.status.is_set(BitIndex::bit_c(0)) {}
+
+        rtc.ctrla.write(match prescaler {
           RtcPrescaler::Div1 => 0x00,
           RtcPrescaler::Div2 => 0x01,
           RtcPrescaler::Div4 => 0x02,
@@ -463,14 +438,13 @@ pub mod base {
           RtcPrescaler::Div16384 => 0x0e,
           RtcPrescaler::Div32768 => 0x0f,
         } << 3);
-
-        v_write!(u8, rtc.clksel, match source {
+        rtc.clksel.write(match source {
           RtcSource::Int32k   => 0x00,
           RtcSource::Int1k    => 0x01,
           RtcSource::TOsc32k  => 0x02,
           RtcSource::External => 0x03
         });
-        v_write!(u8, rtc.calib, match calibration {
+        rtc.calib.write(match calibration {
           RtcCalibration::Fast(val) => {
             val & 0b01111111
           },
@@ -478,7 +452,7 @@ pub mod base {
             0b10000000 | (val & 0b01111111)
           }
         });
-      }
+      });
     }
   }
   impl TimerControl for AvrRealTimeCounter {
@@ -492,11 +466,15 @@ pub mod base {
      *    4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768
      */
     fn set_interrupt_period(&mut self, period: u16) {
-      self.interrupt_period = period.into();
+      interrupt::isolated(|_isotoken|{
+        self.interrupt_period = period.into();
+      });
     }
 
     fn set_mode(&mut self, mode: TimerMode) {
-      self.mode = mode;
+      interrupt::isolated(|_isotoken|{
+        self.mode = mode;
+      });
     }
 
     fn set_count_max(&mut self, _max: u16) {
@@ -504,23 +482,22 @@ pub mod base {
     }
 
     fn start(&self, handler: TimerIsrCallback) {
-      let mut rtc = self.rtc.borrow_mut();
+      interrupt::isolated(|isotoken|{
+        let rtc = self.rtc.borrow_mut(isotoken);
 
-      unsafe {
         // Wait for clock control register to not be busy
-        while (v_read!(u8, rtc.status) & 0x01) != 0 {}
+        while rtc.status.is_set(BitIndex::bit_c(0)) {}
 
         // Set interrupt handler and enable if necessary
-        avr_oxide::hal::concurrency::interrupt::isolated(||{
-          self.interrupt_handler.replace(handler);
-        });
+        self.interrupt_handler.replace(handler);
+
         if handler.is_nop() {
-          v_write!(u8, rtc.pitintctrl, 0);
+          rtc.pitintctrl.clr_all();
         } else {
           // Wait for the periodic interrupt control register to not be busy
-          while (v_read!(u8, rtc.pitstatus) & 0x01) != 0 {}
-          v_write!(u8, rtc.pitctrla,
-            match self.interrupt_period {
+          while rtc.pitstatus.is_set(BitIndex::bit_c(0)) {}
+
+          rtc.pitctrla.write(match self.interrupt_period {
               RtcPitPeriod::Cyc4 => 0x01,
               RtcPitPeriod::Cyc8 => 0x02,
               RtcPitPeriod::Cyc16 => 0x03,
@@ -537,20 +514,21 @@ pub mod base {
               RtcPitPeriod::Cyc32768 => 0x0e,
 
             } << 3 | 0x01);
-          v_write!(u8, rtc.pitintctrl, 1);
+          rtc.pitintctrl.exc_set(BitIndex::bit_c(0));
         }
 
         // Flags: Run In Standby, Enable Correction, Enable RTC
-        v_write!(u8, rtc.ctrla, v_read!(u8, rtc.ctrla) | 0b10000101);
-      }
+        rtc.ctrla |= 0b10000101;
+
+      });
     }
 
     fn stop(&self) {
-      let mut rtc = self.rtc.borrow_mut();
+      interrupt::isolated(|isotoken|{
+        let rtc = self.rtc.borrow_mut(isotoken);
 
-      unsafe {
-        v_write!(u8, rtc.ctrla, v_read!(u8, rtc.ctrla) & 0b01111000);
-      }
+        rtc.ctrla &= 0b01111000;
+      });
     }
 
     fn get_count(&self) -> u16 {
@@ -564,55 +542,51 @@ pub mod base {
 
   impl AvrRealTimeCounter {
     #[inline(always)]
-    pub(crate) fn call_interrupt(&self, source: TimerIdentity, ticks: u16) {
-      let continue_running = isr_cb_invoke!(self.interrupt_handler.get(), source, ticks);
+    pub(crate) fn call_interrupt(&self, isotoken: Isolated, source: TimerIdentity, ticks: u16) {
+      let continue_running = isr_cb_invoke!(isotoken, self.interrupt_handler.get(), source, ticks);
 
       if !continue_running {
         self.stop();
       }
     }
     #[inline(always)]
-    pub(crate) unsafe fn clear_interrupt(&self) {
-      let mut rtc = self.rtc.borrow_mut();
+    pub(crate) unsafe fn clear_interrupt(&self, isotoken: Isolated) {
+      let rtc = self.rtc.borrow_mut(isotoken);
 
-      v_write!(u8, rtc.pitintflags, 0x01);
+      rtc.pitintflags.exc_set(BitIndex::bit_c(0));
     }
   }
 
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_rtc {
-    ($rtcref:expr, $timersrc:expr, $isr: ident) => {
+    ($rtcref:expr, $timersrc:expr, $isr:expr) => {
       use avr_oxide::hal::generic::timer::{ TimerMode, TimerIsrCallback };
       use avr_oxide::hal::generic::timer::base::{ AvrRealTimeCounterControl, AvrRealTimeCounter, RtcPitPeriod };
+      use avr_oxide::hal::generic::MutDeviceRef;
+      use avr_oxide::mut_singleton_explicit_init;
+      use core::cell::Cell;
 
-      use avr_oxide::mut_singleton;
-      use core::cell::{RefCell,Cell};
 
       pub type TimerImpl = AvrRealTimeCounter;
 
-      mut_singleton!(
+      mut_singleton_explicit_init!(
         AvrRealTimeCounter,
-        INSTANCE,
-        instance,
+        __INSTANCE,
+        initialise, instance, instance_isolated,
         AvrRealTimeCounter {
           interrupt_handler: Cell::new(TimerIsrCallback::Nop(false)),
           interrupt_period: RtcPitPeriod::Cyc4,
-          rtc: RefCell::new(core::mem::transmute($rtcref)),
+          rtc: MutDeviceRef::new($rtcref),
           mode: TimerMode::Periodic
         });
 
-      #[no_mangle]
-      pub unsafe extern "avr-interrupt" fn $isr() {
-        avr_oxide::hal::concurrency::interrupt::isr(||{
-          match &mut INSTANCE {
-            None => {},
-            Some(avrrtc) => {
-              avrrtc.call_interrupt($timersrc, 1);
-              avrrtc.clear_interrupt();
-            }
-          }
-        })
+      #[oxide_macros::interrupt(isr=$isr)]
+      fn usart_timer_interrupt(isotoken: avr_oxide::concurrency::Isolated) {
+        let avrrtc = instance_isolated(isotoken);
+
+        avrrtc.call_interrupt(isotoken, $timersrc, 1);
+        avrrtc.clear_interrupt(isotoken);
       }
     }
   }
@@ -620,6 +594,7 @@ pub mod base {
 
 #[cfg(not(target_arch="avr"))]
 pub mod base {
+  use crate::concurrency::Isolated;
   use avr_oxide::hal::generic::timer::{TimerIsrCallback, TimerMode, TimerControl, TimerIdentity, RtcTimerCalibration, RtcSource, RtcCalibration, RtcPrescaler};
   use avr_oxide::isr_cb_invoke;
 
@@ -719,8 +694,8 @@ pub mod base {
       T: DummyTCB
   {
     #[allow(dead_code)]
-    pub(crate) fn call_interrupt(&mut self, source: TimerIdentity, ticks: u16) {
-      let continue_running = isr_cb_invoke!(self.interrupt_handler, source, ticks);
+    pub(crate) fn call_interrupt(&mut self, isotoken: Isolated, source: TimerIdentity, ticks: u16) {
+      let continue_running = isr_cb_invoke!(isotoken, self.interrupt_handler, source, ticks);
 
       if !continue_running {
         self.tcb.disable();
@@ -736,7 +711,7 @@ pub mod base {
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_tcb {
-    ($tcbref:expr, $timersrc:expr, $isr:ident) => {
+    ($tcbref:expr, $timersrc:expr, $isr:expr) => {
       use avr_oxide::hal::generic::timer::{ TimerMode, TimerIsrCallback };
       use avr_oxide::hal::generic::timer::base::{ DummyTimer, DummyTypeBTimerControl };
 
@@ -750,11 +725,19 @@ pub mod base {
         mode: TimerMode::Periodic
       };
 
+      pub fn initialise(){
+      }
+
       pub fn instance() -> &'static mut TimerImpl {
         unsafe {
           &mut INSTANCE
         }
       }
+      pub fn instance_isolated(_isotoken: avr_oxide::concurrency::Isolated) -> &'static mut TimerImpl {
+        unsafe {
+          &mut INSTANCE
+        }
+      }
     }
   }
 
@@ -799,7 +782,7 @@ pub mod base {
         8192 => RtcPitPeriod::Cyc8192,
         16384 => RtcPitPeriod::Cyc16384,
         32768 => RtcPitPeriod::Cyc32768,
-        _ => panic!()
+        _ => avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams)
       }
     }
   }
@@ -856,18 +839,18 @@ pub mod base {
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_rtc {
-    ($rtcref:expr, $timersrc:expr, $isr: ident) => {
+    ($rtcref:expr, $timersrc:expr, $isr:expr) => {
       use avr_oxide::hal::generic::timer::{ TimerMode, TimerIsrCallback };
       use avr_oxide::hal::generic::timer::base::{ DummyRealTimeCounter, RtcPitPeriod };
 
-      use avr_oxide::mut_singleton;
+      use avr_oxide::mut_singleton_explicit_init;
 
       pub type TimerImpl = DummyRealTimeCounter;
 
-      mut_singleton!(
+      mut_singleton_explicit_init!(
         DummyRealTimeCounter,
-        INSTANCE,
-        instance,
+        __INSTANCE,
+        initialise, instance, instance_isolated,
         DummyRealTimeCounter {
           interrupt_handler: TimerIsrCallback::Nop(false),
           interrupt_period: RtcPitPeriod::Cyc4,
