@@ -25,7 +25,13 @@ pub enum IoError {
   NoFreeSpace,
 
   /// Generic IO error
-  Unknown
+  Unknown,
+
+  /// Write not allowed (read-only device)
+  WriteProhibited,
+
+  /// Read not allowed (write-only device)
+  ReadProhibited
 }
 
 pub type Result<T> = core::result::Result<T,IoError>;
