@@ -69,8 +69,34 @@ pub struct VolatileBitField(Volatile<u8>);
  * Index of a bit in a bitfield structure.
  */
 #[derive(Copy,Clone,Eq,PartialEq)]
-#[repr(C)]
-pub struct BitIndex(usize);
+pub struct BitIndex(u8);
+
+/**
+ * Range of bits in a bitfield structure.
+ */
+#[derive(Copy,Clone,Eq,PartialEq)]
+pub struct BitRange {
+  lo: u8,
+  hip1: u8
+}
+
+/// Trait for things that can be turned into a bitmask
+pub trait BitMaskable {
+  /// Return a 'negative' mask (that is, the only bit that is UNset is the
+  /// one corresponding to this bit) for this bit in its byte within the field.
+  ///
+  fn negative_byte_mask(self) -> u8;
+
+  /// Return a 'positive' mask (that is, the only bit that is set is the
+  /// one corresponding to this bit) for this bit in its byte within the field.
+  ///
+  fn positive_byte_mask(self) -> u8;
+
+  /// Return the index of this bit within its byte within the field (i.e.
+  /// the left-shift of this bit)
+  ///
+  fn lo_bit_index(self) -> u8;
+}
 
 pub trait BitFieldAccess {
   fn read_byte(&self) -> u8;
@@ -82,19 +108,34 @@ pub trait BitFieldAccess {
     (self.read_byte() & bit.positive_byte_mask()) > 0
   }
 
+  /// True IFF all the bits at the given index/range are set
+  fn is_all_set<B: BitMaskable+Copy>(&self, bits: B) -> bool {
+    (self.read_byte() & bits.positive_byte_mask()) == bits.positive_byte_mask()
+  }
+
+  /// True IFF any of the bits at the given index/range are set
+  fn is_any_set<B: BitMaskable+Copy>(&self, bits: B) -> bool {
+    (self.read_byte() & bits.positive_byte_mask()) > 0
+  }
+
+  /// True IFF none of the bits at the given index/range are set
+  fn is_all_clr<B: BitMaskable+Copy>(&self, bits: B) -> bool {
+    !self.is_any_set(bits)
+  }
+
   /// True IFF the bit at the given index is clear
   fn is_clr(&self, bit: BitIndex) -> bool {
     !self.is_set(bit)
   }
 
-  /// Set the bit at the given index
-  fn set_isolated(&mut self, _isotoken: Isolated, bit: BitIndex) {
-    self.set(bit)
+  /// Set the bit(s) at the given index/range
+  fn set_isolated<B: BitMaskable+Copy>(&mut self, _isotoken: Isolated, bits: B) {
+    self.set(bits)
   }
 
-  /// Set the bit at the given index
-  fn set(&mut self, bit: BitIndex) {
-    self.write_byte(self.read_byte() | bit.positive_byte_mask());
+  /// Set the bit(s) at the given index/range
+  fn set<B: BitMaskable+Copy>(&mut self, bits: B) {
+    self.write_byte(self.read_byte() | bits.positive_byte_mask());
   }
 
   /// Set all bits of the bitmask to 1
@@ -107,34 +148,51 @@ pub trait BitFieldAccess {
     self.write_byte(0x00);
   }
 
-  /// Set the bit at the given index and clear all others
-  fn exc_set(&mut self, bit: BitIndex) {
-    self.write_byte(bit.positive_byte_mask());
+  /// Set the bit(s) at the given index/range and clear all others
+  fn exc_set<B: BitMaskable+Copy>(&mut self, bits: B) {
+    self.write_byte(bits.positive_byte_mask());
+  }
+
+  /// Clear the bit(s) at the given index/range
+  fn clr_isolated<B: BitMaskable+Copy>(&mut self, _isotoken: Isolated, bits: B) {
+    self.clr(bits)
   }
 
   /// Clear the bit at the given index
-  fn clr_isolated(&mut self, _isotoken: Isolated, bit: BitIndex) {
-    self.clr(bit)
-  }
-
-  /// Clear the bit at the given index
-  fn clr(&mut self, bit: BitIndex) {
-    self.write_byte(self.read_byte() & bit.negative_byte_mask());
+  fn clr<B: BitMaskable+Copy>(&mut self, bits: B) {
+    self.write_byte(self.read_byte() & bits.negative_byte_mask());
   }
 
   /// Clear the bit at the given index and set all others
-  fn exc_clr(&mut self, bit: BitIndex) {
-    self.write_byte(bit.negative_byte_mask());
+  fn exc_clr<B: BitMaskable+Copy>(&mut self, bits: B) {
+    self.write_byte(bits.negative_byte_mask());
   }
 
-  /// Set or clear the bit at the given index depending on the `set`
+  /// Set or clear the bit(s) at the given index/range depending on the `set`
   /// parameter (true = set the bit, false = clear it.)
-  fn set_or_clr(&mut self, bit: BitIndex, set: bool) {
+  fn set_or_clr<B: BitMaskable+Copy>(&mut self, bits: B, set: bool) {
     match set {
-      true  => self.set(bit),
-      false => self.clr(bit)
+      true  => self.set(bits),
+      false => self.clr(bits)
     }
   }
+
+  /// Set the bit(s) at the given index/range to the bitwise value given.
+  fn set_to<B: BitMaskable+Copy>(&mut self, bits: B, value: u8) {
+    let masked = (value << bits.lo_bit_index()) & bits.positive_byte_mask();
+
+    self.write_byte((self.read_byte() & bits.negative_byte_mask()) | masked);
+  }
+
+  /// Set the bit(s) at the given index/range to the bitwise value given.
+  fn set_to_isolated<B: BitMaskable+Copy>(&mut self, _isotoken: Isolated, bits: B, value: u8) {
+    self.set_to(bits, value);
+  }
+
+  /// Return the bit(s) at the given index/range as a binary value.
+  fn get_val<B: BitMaskable+Copy>(&self, bits: B) -> u8 {
+    (self.read_byte() & bits.positive_byte_mask()) >> bits.lo_bit_index()
+  }
 }
 
 // Code ======================================================================
@@ -142,12 +200,12 @@ impl BitIndex {
   /// Construct a BitIndex for the given index
   pub fn bit(b: usize) -> Self {
     #[cfg(feature="runtime_checks")]
-    if b >= 8 {
+    if b > 7 {
       avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams);
     }
 
     Self {
-      0: b
+      0: b as u8
     }
   }
 
@@ -158,34 +216,65 @@ impl BitIndex {
     // baggage - but in a const function, it's OK
 
     Self {
-      0: b
+      0: b as u8
+    }
+  }
+}
+
+impl BitMaskable for BitIndex {
+  fn negative_byte_mask(self) -> u8 {
+    ! self.positive_byte_mask()
+  }
+
+  fn positive_byte_mask(self) -> u8 {
+    0x01u8 << self.lo_bit_index()
+  }
+
+  fn lo_bit_index(self) -> u8 {
+    self.0 as u8
+  }
+}
+
+impl BitRange {
+  pub fn range(lo: usize, hi: usize) -> Self {
+   #[cfg(feature="runtime_checks")]
+   if (lo > 7) || (hi > 7) || (lo > hi) {
+     avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams);
+   }
+
+    Self {
+      lo: lo as u8,
+      hip1: hi as u8 + 1
     }
   }
 
-  /**
-   * Return the index of this bit within its byte within the field.
-   */
-  pub fn bit_index_in_byte(self) -> u8 {
-    self.0 as u8
-  }
+  pub const fn range_c(lo: usize, hi: usize) -> Self {
+    assert!(lo < 8);
+    assert!(hi < 8);
+    assert!(lo <= hi);
 
-  /**
-   * Return a 'positive' mask (that is, the only bit that is set is the
-   * one corresponding to this bit) for this bit in its byte within the field.
-   */
-  pub fn positive_byte_mask(self) -> u8 {
-    0x01u8 << self.bit_index_in_byte()
-  }
-
-  /**
-   * Return a 'negative' mask (that is, the only bit that is UNset is the
-   * one corresponding to this bit) for this bit in its byte within the field.
-   */
-  pub fn negative_byte_mask(self) -> u8 {
-    ! self.positive_byte_mask()
+    Self {
+      lo: lo as u8,
+      hip1: hi as u8 + 1
+    }
   }
 }
 
+impl BitMaskable for BitRange {
+  fn negative_byte_mask(self) -> u8 {
+    ! self.positive_byte_mask()
+  }
+
+  fn positive_byte_mask(self) -> u8 {
+    (0xff << self.hip1) as u8 ^ (0xff << self.lo) as u8
+  }
+
+  fn lo_bit_index(self) -> u8 {
+    self.lo
+  }
+}
+
+
 impl BitField {
   /// Create a BitField initialised to the given values (big-endian)
   pub const fn with_initial(vals: u8) -> Self {
@@ -237,44 +326,58 @@ impl BitFieldAccess for VolatileBitField {
     self.0.read()
   }
 
-  
   fn write_byte(&mut self, val: u8) {
     self.0.write(val);
   }
-
   
-  fn set_isolated(&mut self, _isotoken: Isolated, bit: BitIndex) {
-    self.write_byte(self.read_byte() | bit.positive_byte_mask());
+  fn set_isolated<B: BitMaskable>(&mut self, _isotoken: Isolated, bits: B) {
+    self.write_byte(self.read_byte() | bits.positive_byte_mask());
   }
 
-  /// Set the bit at the given index.  Since this is a R-M-W method, we do it
-  /// with interrupts disabled for volatile fields.
-  fn set(&mut self, bit: BitIndex) {
+  fn set<B: BitMaskable+Copy>(&mut self, bits: B) {
     avr_oxide::concurrency::interrupt::isolated(|isotoken|{
-      self.set_isolated(isotoken, bit)
+      self.set_isolated(isotoken, bits)
     })
   }
 
-  
-  fn clr_isolated(&mut self, _isotoken: Isolated, bit: BitIndex) {
-    self.write_byte(self.read_byte() & bit.negative_byte_mask());
+  fn clr_isolated<B: BitMaskable+Copy>(&mut self, _isotoken: Isolated, bits: B) {
+    self.write_byte(self.read_byte() & bits.negative_byte_mask());
   }
 
-  /// Clear the bit at the given index.  Since this is a R-M-W method, we do it
-  /// with interrupts disabled for volatile fields.
-  fn clr(&mut self, bit: BitIndex) {
+  fn clr<B: BitMaskable+Copy>(&mut self, bits: B) {
     avr_oxide::concurrency::interrupt::isolated(|isotoken|{
-      self.clr_isolated(isotoken, bit)
+      self.clr_isolated(isotoken, bits)
     });
   }
+
+  fn set_or_clr<B: BitMaskable+Copy>(&mut self, bits: B, set: bool) {
+    avr_oxide::concurrency::interrupt::isolated(|isotoken| {
+      match set {
+        true  => self.set_isolated(isotoken, bits),
+        false => self.clr_isolated(isotoken, bits)
+      }
+    })
+  }
+
+  fn set_to<B: BitMaskable+Copy>(&mut self, bits: B, value: u8) {
+    avr_oxide::concurrency::interrupt::isolated(|isotoken| {
+      self.set_to_isolated(isotoken, bits, value)
+    })
+  }
+
+  fn set_to_isolated<B: BitMaskable+Copy>(&mut self, _isotoken: Isolated, bits: B, value: u8) {
+    let masked = (value << bits.lo_bit_index()) & bits.positive_byte_mask();
+
+    self.write_byte((self.read_byte() & bits.negative_byte_mask()) | masked);
+  }
 }
 
 impl VolatileBitField {
-  pub fn all_clr() -> Self {
+  pub const fn all_clr() -> Self {
     VolatileBitField(Volatile::<u8>::zero())
   }
-  pub fn all_set() -> Self {
-    VolatileBitField(Volatile::<u8>::from(0xff))
+  pub const fn all_set() -> Self {
+    VolatileBitField(Volatile::<u8>::effeff())
   }
 
 
@@ -294,6 +397,10 @@ impl Volatile<u8> {
   pub const fn zero() -> Self {
     Volatile(0)
   }
+
+  pub const fn effeff() -> Self {
+    Volatile(0xff)
+  }
   
   pub fn snapshot(&self) -> u8 {
     unsafe {
@@ -606,16 +713,25 @@ volatile_multibyte_impl!(usize);
 // Tests =====================================================================
 #[cfg(test)]
 mod tests {
-  use avr_oxide::util::datatypes::{BitField, BitFieldAccess, BitIndex, BoundedIncDec, BoundedMaths, Volatile, VolatileBitField};
+  use avr_oxide::util::datatypes::{BitField, BitFieldAccess, BitIndex, BitMaskable, BitRange, BoundedIncDec, BoundedMaths, Volatile, VolatileBitField};
 
   #[test]
   fn test_bitindex() {
     let bit4 = BitIndex(4);
-    assert_eq!(bit4.bit_index_in_byte(), 4);
+    assert_eq!(bit4.lo_bit_index(), 4);
     assert_eq!(bit4.positive_byte_mask(), 0b00010000);
     assert_eq!(bit4.negative_byte_mask(), 0b11101111);
   }
 
+  #[test]
+  fn test_bitrange() {
+    let bit2to6 = BitRange::range(2,6);
+
+    assert_eq!(bit2to6.lo_bit_index(), 2);
+    assert_eq!(bit2to6.positive_byte_mask(), 0b01111100);
+    assert_eq!(bit2to6.negative_byte_mask(), 0b10000011);
+  }
+
   #[test]
   fn test_bitfield() {
     let mut all_zero : BitField = BitField::all_clr();
@@ -640,6 +756,17 @@ mod tests {
     assert!(all_one.is_clr(BitIndex(7)));
     assert!(!all_zero.is_set(BitIndex(0)));
     assert!(!all_one.is_clr(BitIndex(1)));
+
+    all_zero.set(BitRange::range(5,7));
+    all_one.clr(BitRange::range_c(2,4));
+
+    assert!(all_zero == BitField::with_initial(0b11101000));
+    assert!(all_one  == BitField::with_initial(0b01100011));
+
+    assert!(all_zero.get_val(BitRange::range_c(1,5)) == 0b10100);
+
+    all_one.set_to(BitRange::range(0,3), 0b1010);
+    assert!(all_one  == BitField::with_initial(0b01101010));
   }
 
   #[test]
@@ -666,6 +793,17 @@ mod tests {
     assert!(all_one.is_clr(BitIndex(7)));
     assert!(!all_zero.is_set(BitIndex(0)));
     assert!(!all_one.is_clr(BitIndex(1)));
+
+    all_zero.set(BitRange::range(5,7));
+    all_one.clr(BitRange::range_c(2,4));
+
+    assert!(all_zero.snapshot() == BitField::with_initial(0b11101000));
+    assert!(all_one.snapshot()  == BitField::with_initial(0b01100011));
+
+    assert!(all_zero.get_val(BitRange::range(1,5)) == 0b10100);
+
+    all_one.set_to(BitRange::range_c(0,3), 0b1010);
+    assert!(all_one.snapshot()  == BitField::with_initial(0b01101010));
   }
 
   #[test]
